@@ -323,8 +323,7 @@ setorderv <- function(x, cols, order=1L, na.last=FALSE)
   if (length(o)) {
     .Call(Creorder, x, o)
     if (is.data.frame(x) & !is.data.table(x)) {
-      .Call(Creorder, rn <- rownames(x), o)
-      setattr(x, 'row.names', rn)
+      setattr(x, 'row.names', rownames(x)[o])
     }
     setattr(x, 'sorted', NULL) # if 'forderv' is not 0-length, it means order has changed. So, set key to NULL, else retain key.
     setattr(x, 'index', NULL)  # remove secondary keys too. These could be reordered and retained, but simpler and faster to remove
