@@ -26,6 +26,7 @@ SEXP setlistelt();
 SEXP setmutable();
 SEXP address();
 SEXP copyNamedInList();
+SEXP expandAltRep();
 SEXP fmelt();
 SEXP fcast();
 SEXP uniqlist();
@@ -105,6 +106,7 @@ R_CallMethodDef callMethods[] = {
 {"Csetmutable", (DL_FUNC) &setmutable, -1},
 {"Caddress", (DL_FUNC) &address, -1},
 {"CcopyNamedInList", (DL_FUNC) &copyNamedInList, -1},
+{"CexpandAltRep", (DL_FUNC) &expandAltRep, -1},
 {"Cfmelt", (DL_FUNC) &fmelt, -1},
 {"Cfcast", (DL_FUNC) &fcast, -1},
 {"Cuniqlist", (DL_FUNC) &uniqlist, -1},
